/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.charts;

import org.apache.commons.lang.StringUtils;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.SpiderWebPlot;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.sonar.plugins.api.charts.AbstractChart;
import org.sonar.plugins.api.charts.ChartParameters;

import java.awt.*;
import java.awt.image.BufferedImage;

public class XradarChart extends AbstractChart {

  /**
   * see an example of complete URL in XradarChartTest
   */

  public static final String PARAM_COLOR = "c";
  public static final String PARAM_MAX_VALUE = "m";
  public static final String PARAM_INTERIOR_GAP = "g";
  public static final String PARAM_LABELS = "l";
  public static final String PARAM_VALUES = "v";

  public String getKey() {
    return "xradar";
  }

  public BufferedImage generateImage(ChartParameters params) {
    SpiderWebPlot plot = configurePlot(params);
    JFreeChart chart = new JFreeChart(null, TextTitle.DEFAULT_FONT, plot, false);
    return generateImage(chart, params);
  }

  private SpiderWebPlot configurePlot(ChartParameters params) {
    SpiderWebPlot plot = new SpiderWebPlot(createDataset(params));
    plot.setStartAngle(0D);
    plot.setOutlineVisible(false);
    plot.setAxisLinePaint(Color.decode("0xCCCCCC"));
    plot.setSeriesOutlineStroke(new BasicStroke(2f));

    if (params.getValue(PARAM_INTERIOR_GAP) != null) {
      plot.setInteriorGap(Double.parseDouble(params.getValue(PARAM_INTERIOR_GAP, "0.4")));
    }
    if (params.getValue(PARAM_MAX_VALUE) != null) {
      plot.setMaxValue(Double.parseDouble(params.getValue(PARAM_MAX_VALUE, "100")));
    }
    configureColors(plot, params);
    return plot;
  }

  private void configureColors(SpiderWebPlot plot, ChartParameters params) {
    String[] colors = params.getValues(PARAM_COLOR, "|");
    for (int i = 0; i < colors.length; i++) {
      plot.setSeriesPaint(i, Color.decode("0x" + colors[i]));
    }
  }

  private CategoryDataset createDataset(ChartParameters params) {
    String[] labels = params.getValues(PARAM_LABELS, ",");
    String[] values = params.getValues(PARAM_VALUES, "|");

    DefaultCategoryDataset set = new DefaultCategoryDataset();
    for (int indexValues = 0; indexValues < values.length; indexValues++) {
      String[] fields = StringUtils.split(values[indexValues], ",");
      for (int i = 0; i < fields.length; i++) {
        set.addValue(Double.parseDouble(fields[i]), "" + indexValues, labels[i]);
      }
    }
    return set;
  }
}