/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.clouds.client;

import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.http.client.URL;
import com.google.gwt.json.client.JSONArray;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Panel;
import org.sonar.plugins.api.web.gwt.client.AbstractPage;
import org.sonar.plugins.api.web.gwt.client.JsonUtils;
import org.sonar.plugins.api.web.gwt.client.ResourceDictionary;
import org.sonar.plugins.api.web.gwt.client.Utils;
import org.sonar.plugins.api.web.gwt.client.rest.RestUtils;
import org.sonar.plugins.api.web.gwt.client.rest.model.Project;
import org.sonar.plugins.core.clouds.client.widget.ClassCloudsWidget;
import org.sonar.plugins.core.clouds.client.widget.TabWidget;

import java.util.List;

public class GwtClouds extends AbstractPage {

  public static final String GWT_ID = "org.sonar.plugins.core.clouds.GwtClouds";

  private ResourceDictionary resourceDictionary = new ResourceDictionary();
  private RestUtils restApi = new RestUtils();

  private Panel cloudsPanel = new FlowPanel();

  // Warning : changing metrics and its position change tabs
  private static final String METRICS = "ccn_function,ccn,code_coverage";

  public void onModuleLoad() {
    displayView(cloudsPanel);
    loadClouds();
  }

  protected void loadClouds() {
    String projectKey = resourceDictionary.getPageProjectKey();
    if (projectKey != null) {
      String restUrl = Utils.getServerUrl() + "/api/projects/" + projectKey + "/measures?format=json&verbose=full&scope=CLA&metrics=" + METRICS;
      String url = URL.encode(restUrl) + "&callback=";
      JsonUtils.requestJson(url, new CloudsHandler());
    }
  }

  public class CloudsHandler implements JsonUtils.JSONHandler {
    public void handleJSON(JavaScriptObject jso) {
      if (jso == null) {
        Utils.showError("Couldn't retrieve JSON measures");
        return;
      }
      List<Project> projects = restApi.processResponse(new JSONArray(jso));
      updateClouds(projects);
    }
  }

  private void updateClouds(List<Project> projects) {
    cloudsPanel.clear();
    TabWidget tab = new TabWidget();
    ClassCloudsWidget firstTab = new ClassCloudsWidget(projects, getMetric(0), getMetric(2), tab);
    ClassCloudsWidget secondTab = new ClassCloudsWidget(projects, getMetric(1), getMetric(2), tab);
    tab.addTab(firstTab, "Project risk", "project_risk");
    tab.addTab(secondTab, "Quick Wins", "quick_wins");
    cloudsPanel.add(tab);
  }

  private String getMetric(int position) {
    return METRICS.split(",")[position];
  }

}