/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.codecoveragesourceviewer.client;

import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.http.client.URL;
import com.google.gwt.json.client.JSONArray;
import com.google.gwt.json.client.JSONValue;
import org.sonar.plugins.api.web.gwt.client.AbstractSourceViewer;
import org.sonar.plugins.api.web.gwt.client.JsonUtils;
import org.sonar.plugins.api.web.gwt.client.LineDecorator;
import org.sonar.plugins.api.web.gwt.client.Utils;

import java.util.HashMap;
import java.util.Map;

public class GwtCodeCoverageSourceViewer extends AbstractSourceViewer {
  public static final String GWT_ID = "org.sonar.plugins.core.codecoveragesourceviewer.GwtCodeCoverageSourceViewer";


  public GwtCodeCoverageSourceViewer() {
    setExpandLinkActivation(false);
  }

  protected boolean showValueColumn() {
    return true;
  }

  protected void requestLineDecorators(String sourceKey) {
    // 'code_coverage' can not be replaced by CoreMetrics.CODE_COVERAGE.getKey() for GWT compilation reasons
    String url = URL.encode(Utils.getServerUrl() + "/api/projects/" + sourceKey + "/measures?metrics=code_coverage&includeparams=true&format=json&callback=");
    JsonUtils.requestJson(url, new LineDecoratorsHandler());
  }

  private String getLinesValue(JSONArray rootArray) {
    JSONValue lines = null;
    JSONArray measures = rootArray.get(0).isObject().get("measures").isArray();
    if (measures.size() > 0) {
      JSONArray msr = measures.get(0).isObject().get("msr").isArray();
      if (msr.size() > 0) {
        JSONValue prm = msr.get(0).isObject().get("params");
        if (prm!=null && prm.isArray().size() > 0) {
          lines = prm.isArray().get(0).isObject().get("lob_value");
          if (lines!=null && lines.isString()!=null) {
            return lines.isString().toString();
          }
        }
      }
    }
    return null;
  }

  public class LineDecoratorsHandler implements JsonUtils.JSONHandler {

    public void handleJSON(JavaScriptObject obj) {
      if (obj == null) {
        return;
      }
      Map<Integer, LineDecorator> result = new HashMap<Integer, LineDecorator>();
      JSONArray jsonArray = new JSONArray(obj);
      String linesValue = getLinesValue(jsonArray);
      // remove first '"' and last ',"'
      if (linesValue!=null) {
        linesValue = linesValue.substring(1, linesValue.length() - 2);
        String[] lineWithValueArray = linesValue.split(",");
        for (String lineWithValue : lineWithValueArray) {
          String[] elt = lineWithValue.split("=");
          int line = Integer.parseInt(elt[0]);
          String value = elt[1];
          if (Integer.parseInt(value) == 0) {
            result.put(line, new LineDecorator(AbstractSourceViewer.ERROR_STYLE, value, AbstractSourceViewer.ERROR_STYLE, null));
          } else if (Integer.parseInt(value) > 0) {
            result.put(line, new LineDecorator(null, value, AbstractSourceViewer.INFO_STYLE, null));
          }
        }
      }
      setDecorators(result);
      setDecoratorsRetrieved(true);
      refreshLines();
    }
  }

  protected void exportJavascript() {
    exportNativeJavascript(this);
  }

  protected String getGwtId() {
    return GWT_ID;
  }


  public static native void exportNativeJavascript(Object obj) /*-{
    $wnd.load_org_sonar_plugins_core_codecoveragesourceviewer_GwtCodeCoverageSourceViewer = function() {
      obj.@org.sonar.plugins.core.codecoveragesourceviewer.client.GwtCodeCoverageSourceViewer::loadSource()();
    }
  }-*/;

}