/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.complexity;

import org.sonar.commons.measures.Measure;
import org.apache.commons.collections.bag.HashBag;
import org.apache.commons.lang.StringUtils;

class CountDistribution implements Distribution {
  static final int[] BOTTOM_LIMITS = {0, 5, 10, 20, 30, 60, 90};

  private HashBag countByRangeIndex = new HashBag();

  public void add(Measure measure) {
    if (measure != null) {
      countByRangeIndex.add(getRangeIndex(measure.getValue()));
    }
  }

  public String asString() {
    Range[] ranges = new Range[BOTTOM_LIMITS.length];
    for (int index = 0; index < BOTTOM_LIMITS.length; index++) {
      ranges[index] = new Range(BOTTOM_LIMITS[index], countByRangeIndex.getCount(index));
    }

    return StringUtils.join(ranges, ';');
  }

  private static Integer getRangeIndex(Double value) {
    for (int i = BOTTOM_LIMITS.length - 1; i > 0; i--) {
      if (value >= BOTTOM_LIMITS[i]) {
        return i;
      }
    }
    return 0;
  }
}
