/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.complexity;

import org.sonar.commons.measures.Measure;
import org.apache.commons.lang.StringUtils;

class PercentDistribution implements Distribution {
  private Range[] percentByRange;

  public void add(Measure measure) {
    if (measure == null || measure.getTextValue() == null) {
      return;
    }
    Range[] ranges = parse(measure);
    int total = total(ranges);
    calculatePercentDistribution(ranges, total);
  }

  public String asString() {
    if (percentByRange == null) {
      return null;
    }
    String[] formattedPercent = getFormattedPercentByRange();
    return StringUtils.join(formattedPercent, ';');
  }

  private String[] getFormattedPercentByRange() {
    String[] formattedPercent = new String[percentByRange.length];
    for (int i = 0; i < formattedPercent.length; i++) {
      formattedPercent[i] = percentByRange[i].toString();
    }
    return formattedPercent;
  }

  private void calculatePercentDistribution(Range[] ranges, int total) {
    percentByRange = new Range[ranges.length];
    for (int i = 0; i < percentByRange.length; i++) {
      if (total > 0.0){
        percentByRange[i] = new Range(ranges[i].getLowerLimit(), ranges[i].getValue().intValue() * 100.0 / total);
      } else {
        percentByRange[i] = new Range(ranges[i].getLowerLimit(), 0);
      }

    }
  }

  private static int total(Range[] ranges) {
    int result = 0;
    for (Range range : ranges) {
      result += range.getValue().intValue();
    }
    return result;
  }

  private static Range[] parse(Measure measure) {
    String[] parts = StringUtils.split(measure.getTextValue(), ';');
    Range[] result = new Range[parts.length];
    for (int i = 0; i < parts.length; i++) {
      result[i] = Range.parse(parts[i]);
    }
    return result;
  }
}
