/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.complexity;

import org.apache.commons.lang.StringUtils;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

public class Range implements Comparable {
  private int lowerLimit;
  private Number value;

  public Range(int lowerLimit) {
    this.lowerLimit = lowerLimit;
  }

  public Range(int lowerLimit, Number value) {
    this.lowerLimit = lowerLimit;
    this.value = value;
  }

  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;

    Range range = (Range) o;
    return (lowerLimit == range.lowerLimit);
  }

  public int hashCode() {
    return lowerLimit;
  }

  public String toString() {
    StringBuilder sb = new StringBuilder();
    sb.append(lowerLimit);
    sb.append("=");
    if (value instanceof Double) {
      sb.append(createDecimalFormatter().format(value));
    } else {
      sb.append(value);
    }
    return sb.toString();
  }

  public int getLowerLimit() {
    return lowerLimit;
  }

  public void setLowerLimit(int lowerLimit) {
    this.lowerLimit = lowerLimit;
  }

  public Number getValue() {
    return value;
  }

  public void setValue(Number value) {
    this.value = value;
  }

  public static Range parse(String s) {
    String[] strings = StringUtils.split(s, "=");
    try {
      return new Range(Integer.parseInt(strings[0]), createDecimalFormatter().parse(strings[1]));
    } catch (ParseException e) {
      throw new IllegalArgumentException(e);
    }
  }

  private static NumberFormat createDecimalFormatter() {
    NumberFormat formatter = NumberFormat.getInstance(Locale.US);
    formatter.setMaximumFractionDigits(2);
    return formatter;
  }

  public int compareTo(Object o) {
    return lowerLimit - ((Range) o).getLowerLimit();
  }
}
