/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.complexity;

import ch.hortis.sonar.model.Metric;
import org.sonar.commons.measures.Measure;
import org.sonar.plugins.api.jobs.Resource;
import org.sonar.plugins.api.jobs.Job;
import org.sonar.plugins.api.metrics.CoreMetrics;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class SumComplexityCountDistributionJob implements Job {

  public List<Metric> dependsOnMetrics() {
    return Collections.emptyList();
  }

  public List<Class<? extends Job>> dependsOnJobs() {
    return Collections.emptyList();
  }

  public List<Metric> generatesMetrics() {
    return Arrays.asList(CoreMetrics.COMPLEXITY_CLASSES_COUNT_DISTRIBUTION);
  }

  public boolean shouldExecuteOnProject(Resource project) {
    return true;
  }

  public boolean shouldExecuteOnResource(Resource node) {
    return !(node.isClass() || node.isPackage());
  }

  public void execute(Resource node) {
    Distribution distribution = new SumCountDistribution();

    for (Measure complexityMeasure : node.getChildrenMeasures(CoreMetrics.COMPLEXITY_CLASSES_COUNT_DISTRIBUTION)) {
      distribution.add(complexityMeasure);
    }

    node.addMeasure(CoreMetrics.COMPLEXITY_CLASSES_COUNT_DISTRIBUTION, distribution.asString());
  }
}