/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.complexity;

import org.sonar.commons.measures.Measure;
import org.apache.commons.lang.StringUtils;

import java.util.Map;
import java.util.TreeMap;

class SumCountDistribution implements Distribution {
  private Map<Integer, Range> rangeByLimit = new TreeMap<Integer, Range>();

  public void add(Measure measure) {
    if (measure == null || measure.getTextValue() == null) {
      return;
    }
    boolean firstAdd = rangeByLimit.size() == 0;
    String[] parts = StringUtils.split(measure.getTextValue(), ';');
    for (String part : parts) {
      Range range = Range.parse(part);
      Range sumRange = rangeByLimit.get(range.getLowerLimit());
      if (sumRange == null) {
        if (!firstAdd) {
          throw new IllegalArgumentException("Can't compute distributions with different ranges");
        }

        rangeByLimit.put(range.getLowerLimit(), range);
      } else {
        sumRange.setValue(sumRange.getValue().doubleValue() + range.getValue().doubleValue());
      }
    }
  }

  public String asString() {
    if (rangeByLimit.size() == 0) {
      return null;
    }
    return StringUtils.join(rangeByLimit.values(), ';');
  }
}