/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.events;

import org.sonar.commons.events.Event;
import org.sonar.plugins.api.Language;
import org.sonar.plugins.api.jobs.AbstractJob;
import org.sonar.plugins.api.jobs.Resource;
import org.apache.commons.lang.time.DateUtils;

import java.util.Iterator;

public class SetAutomaticEventsJob extends AbstractJob {
  private static final String SNAPSHOT_SUFFIX = "-SNAPSHOT";

  protected Language shouldExecuteOnLanguage() {
    // all languages
    return null;
  }

  public boolean shouldExecuteOnResource(Resource resource) {
    return resource.isProject();
  }

  public void execute(Resource resource) {
    if (resource.getVersion()==null) {
      return;
    }

    deleteDeprecatedEvents(resource);
    resource.createEventOnCurrentSnapshot(resource.getVersion(), null, Event.CATEGORY_VERSION);
  }

  private void deleteDeprecatedEvents(Resource resource) {
    String version=resource.getVersion();
    String snapshotVersionToDelete = (version.endsWith(SNAPSHOT_SUFFIX) ? "" : version + SNAPSHOT_SUFFIX);
    for (Iterator<Event> it = resource.getEvents().iterator(); it.hasNext();) {
      Event event=it.next();
      if (event.isVersionCategory()) {
        if (version.equals(event.getName()) || snapshotVersionToDelete.equals(event.getName())) {
          it.remove();
          resource.deleteEvent(event);
          event=null;
        }
      }

      if (event!=null && !event.isLinkedToSnapshot() &&
          event.getDate() != null && resource.getDate()!=null &&
          DateUtils.isSameDay(event.getDate(), resource.getDate())) {
        it.remove();
        resource.deleteEvent(event);
        resource.createEventOnCurrentSnapshot(event.getName(), event.getDescription(), event.getCategory());
      }
    }
  }
}
