/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.resourceviewer.client;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.http.client.URL;
import com.google.gwt.json.client.JSONArray;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.RootPanel;
import org.sonar.plugins.api.web.gwt.client.JsonUtils;
import org.sonar.plugins.api.web.gwt.client.ResourceDictionary;
import org.sonar.plugins.api.web.gwt.client.Utils;
import org.sonar.plugins.api.web.gwt.client.rest.RestUtils;
import org.sonar.plugins.api.web.gwt.client.rest.model.Measure;
import org.sonar.plugins.api.web.gwt.client.rest.model.Project;

import java.util.List;
import java.util.Map;

public class GwtResourceViewer implements EntryPoint {

  public static final String HTML_ROOT_ID = "resourceviewer";
  public static final String GWT_ID = "org.sonar.plugins.core.resourceviewer.GwtResourceViewer";
  protected static final String METRICS_TO_RETRIEVE = "ncss,functions_count,duplicated_lines,ccn,ccn_function,code_coverage,rules_violations,optional_rules_violations";

  private Panel container = null;
  private Label title = null;
  private HeaderWidget header = new HeaderWidget();

  private ResourceDictionary resourceDictionary = new ResourceDictionary();
  private RestUtils restApi = new RestUtils();

  private String classKey = "";

  public void onModuleLoad() {
    header = new HeaderWidget();
    exportJavascript();
    loadContainer();
  }

  protected void loadContainer() {
    if (container == null) {
      RootPanel rootPanel = RootPanel.get(HTML_ROOT_ID);
      if (rootPanel != null) {
        title = new Label();
        title.setStyleName("h1 title");

        container = new FlowPanel();
        container.add(title);
        container.add(header);
        rootPanel.add(container);

        clearOnlyHeader();
      }
    }
  }

  protected void loadSource() {
    loadContainer();
    String newSourceKey = resourceDictionary.getSourceKey();
    if (newSourceKey != null) {
      clearOnlySourceViewer();
      if (!classKey.equals(newSourceKey) || !container.isVisible()) {
        classKey = newSourceKey;
        clearOnlyHeader();
        refreshResourceViewer();
      } else {
        refreshSourceViewer(resourceDictionary.getPluginKey());
      }
    }
  }

  private void refreshResourceViewer() {
    String classKey = resourceDictionary.getSourceKey();
    String restUrl = Utils.getServerUrl() + "/api/projects/" + classKey + "/measures?format=json&verbose=full&metrics=" + METRICS_TO_RETRIEVE;
    String url = URL.encode(restUrl) + "&callback=";
    JsonUtils.requestJson(url, new HeaderHandler());
  }

  private void clearOnlyHeader() {
    container.setVisible(false);
    title.setText(null);
    header.init();
  }

  public class HeaderHandler implements JsonUtils.JSONHandler {
    public void handleJSON(JavaScriptObject jso) {
      if (jso == null) {
        Utils.showError("Couldn't retrieve JSON measures");
        return;
      }
      List<Project> projects = restApi.processResponse(new JSONArray(jso));
      if (restApi.hasProjectAtLeastOneMeasure(projects)) {
        updateResourceViewer(projects);
      }
    }
  }

  private void updateResourceViewer(List<Project> projects) {
    Map<String, Measure> measures = projects.get(0).getSnapshots().get(0).getMeasures();
    title.setText(projects.get(0).getName());
    header.updateMeasures(measures);
    container.setVisible(true);

    refreshSourceViewer(resourceDictionary.getPluginKey());
  }


  protected void exportJavascript() {
    exportNativeJavascript(this);
  }

  public static native void exportNativeJavascript(Object obj) /*-{
    $wnd.load_org_sonar_plugins_core_resourceviewer_GwtResourceViewer = function() {
      obj.@org.sonar.plugins.core.resourceviewer.client.GwtResourceViewer::loadSource()();
    };
  }-*/;

  private native void clearOnlySourceViewer()/*-{
    $wnd.$$('#sourceviewers .sourceviewer').invoke('hide');
  }-*/;

  private native void refreshSourceViewer(String pluginKey) /*-{
    $wnd.config['sourceviewer_plugin_key'] = pluginKey || 'org.sonar.plugins.core.defaultsourceviewer.GwtDefaultSourceViewer';
    $wnd.callGwtViewer($wnd.config['sourceviewer_plugin_key']);
  }-*/;
}