/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.resourceviewer.client;

import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.Label;
import org.sonar.plugins.api.web.gwt.client.SonarWidget;
import org.sonar.plugins.api.web.gwt.client.rest.model.Measure;

import java.util.HashMap;
import java.util.Map;

public class HeaderWidget extends SonarWidget {

  private static final String BLOCK_ID = "resource_header_";
  private Map<String, Label> labelById = new HashMap<String, Label>();
  private Grid main;

  public HeaderWidget() {
    main = new Grid(3, 8);
    init();

    initWidget(main);
    setStyleName("header");
  }

  public void init(){
    main.setWidth("100%");
    main.setCellSpacing(5);

    main.setWidget(0, 0, createPanel(getMetrics()[0], Type.LABEL));
    main.setWidget(0, 1, createPanel(getMetrics()[0], Type.VALUE));

    main.setWidget(1, 0, createPanel(getMetrics()[1], Type.LABEL));
    main.setWidget(1, 1, createPanel(getMetrics()[1], Type.VALUE));

    main.setWidget(2, 0, createPanel(getMetrics()[2], Type.LABEL));
    main.setWidget(2, 1, createPanel(getMetrics()[2], Type.VALUE));

    main.setWidget(0, 2, createPanel(getMetrics()[3], Type.LABEL));
    main.setWidget(0, 3, createPanel(getMetrics()[3], Type.VALUE));

    main.setWidget(1, 2, createPanel(getMetrics()[4], Type.LABEL));
    main.setWidget(1, 3, createPanel(getMetrics()[4], Type.VALUE));

    main.setWidget(0, 4, createPanel(getMetrics()[5], Type.LABEL));
    main.setWidget(0, 5, createPanel(getMetrics()[5], Type.VALUE));

    main.setWidget(0, 6, createPanel(getMetrics()[6], Type.LABEL));
    main.setWidget(0, 7, createPanel(getMetrics()[6], Type.VALUE));

    main.setWidget(1, 6, createPanel(getMetrics()[7], Type.LABEL));
    main.setWidget(1, 7, createPanel(getMetrics()[7], Type.VALUE));
  }

  private String[] getMetrics() {
    return GwtResourceViewer.METRICS_TO_RETRIEVE.split(",");
  }

  private Label createPanel(String metric, Type type) {
    Label label = new Label();
    if (type == Type.LABEL) {
      label.setStyleName("label");
    } else {
      label.setStyleName("value");
    }
    String labelId = BLOCK_ID + metric + "_" + type.text;
    label.getElement().setId(labelId);
    labelById.put(labelId, label);

    return label;
  }


  public void updateMeasures(Map<String, Measure> projectMeasures) {
    for (String metricKey : getMetrics()) {
      Measure measure = projectMeasures.get(metricKey);
      if (measure != null) {
        String metricName = measure.getMetricName();
        String value = measure.getFormattedValue();

        Label metricLabel = labelById.get(BLOCK_ID + metricKey + "_label");
        metricLabel.setText(metricName + ":");

        Label metricValue = labelById.get(BLOCK_ID + metricKey + "_value");
        metricValue.setText(value);
      }
    }
  }

  enum Type {
    LABEL("label"), VALUE("value");

    String text;

    Type(String text) {
      this.text = text;
    }
  }
}
