/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.rules;

import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.service.MeasureKey;
import org.apache.commons.configuration.Configuration;
import org.sonar.commons.measures.Measure;
import org.sonar.plugins.api.jobs.Job;
import org.sonar.plugins.api.jobs.Resource;
import org.sonar.plugins.api.metrics.CoreMetrics;
import org.sonar.plugins.core.CorePlugin;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public abstract class AbstractRespectJob implements Job {
  public List<Metric> dependsOnMetrics() {
    return Arrays.asList(CoreMetrics.NLOC, getCountMetric());
  }

  public List<Class<? extends Job>> dependsOnJobs() {
    return Collections.emptyList();
  }

  public List<Metric> generatesMetrics() {
    return Arrays.asList(getRespectMetric());
  }

  public AbstractRespectJob(Configuration configuration) {
    nbLinesForOneError = configuration.getInt(CorePlugin.PROP_KEY_CRC_FACTOR, CorePlugin.PROP_DEFAULT_CRC_FACTOR);
  }

  protected AbstractRespectJob() {
  }

  public boolean shouldExecuteOnProject(Resource project) {
    return true;
  }

  public boolean shouldExecuteOnResource(Resource resource) {
    return !resource.isJavaTest();
  }


  private int nbLinesForOneError = CorePlugin.PROP_DEFAULT_CRC_FACTOR;

  public int getNbLinesForOneError() {
    return nbLinesForOneError;
  }

  public void setNbLinesForOneError(int nbLinesForOneError) {
    this.nbLinesForOneError = nbLinesForOneError;
  }

  protected abstract Metric getCountMetric();

  protected abstract Metric getRespectMetric();


  public void execute(Resource node) {
    Measure ncss = node.getMeasure(CoreMetrics.NLOC);
    if (ncss != null) {
      Double lines = ncss.getValue();
      if (lines > 0.0) {
        List<Measure> violationMeasures = node.getMeasures(getCountMetric());
        for (Measure measure : violationMeasures) {
          MeasureKey key = new MeasureKey(getRespectMetric(), measure.getRulesCategoryId(), measure.getRule());
          (node).addMeasure(key, getValue(measure.getValue(), lines, nbLinesForOneError));
        }
      }
    }
  }

  protected double getValue(Double numberOfFailures, Double numberOfLines, int nbLinesForOneError) {
    return 100.0 - Math.min(100.0, (numberOfFailures * 100.0 * nbLinesForOneError) / numberOfLines);
  }

}
