package org.sonar.plugins.core.testspage.client;

import com.google.gwt.json.client.JSONArray;
import com.google.gwt.json.client.JSONObject;
import com.google.gwt.json.client.JSONString;
import com.google.gwt.json.client.JSONValue;
import org.sonar.plugins.api.web.gwt.client.AbstractPage;
import org.sonar.plugins.api.web.gwt.client.Utils;
import org.sonar.plugins.core.testspage.client.model.Measure;
import org.sonar.plugins.core.testspage.client.model.Parameter;
import org.sonar.plugins.core.testspage.client.model.Project;
import org.sonar.plugins.core.testspage.client.model.Snapshot;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public abstract class AbstractMeasuresPage extends AbstractPage {


  public String getProjectKey(){
    return Utils.getConfiguration("page_project_key");
  }

  protected String buildRestUrl(String metrics) {
    return Utils.getServerUrl() + "/api/projects/" + getProjectKey() + "/measures?format=json&metrics=" + metrics;
  }

  protected native void displayError(String message) /*-{
    $wnd.error(message);
  }-*/;

  protected ArrayList<Project> processResponse(JSONArray array) {
    ArrayList<Project> projects = new ArrayList<Project>();
    for (int i = 0; i < array.size(); i++) {
      JSONObject jsStock;
      if ((jsStock = array.get(i).isObject()) == null) continue;
      Project project = processProject(jsStock);
      projects.add(project);
    }
    return projects;
  }

  private Project processProject(JSONObject jsStock) {
    JSONValue jsonValue;
    JSONString key = null;
    JSONString name = null;
    JSONString qualifier = null;

    jsonValue = jsStock.get("key");

    if ((jsonValue = jsStock.get("key")) != null) {
      key = jsonValue.isString();
    }

    if ((jsonValue = jsStock.get("name")) != null) {
      name = jsonValue.isString();
    }

    if ((jsonValue = jsStock.get("qualifier")) != null) {
      qualifier = jsonValue.isString();
    }

    ArrayList<Snapshot> snapshots = null;
    if ((jsonValue = jsStock.get("measures")) != null) {
      JSONArray measures = jsonValue.isArray();

      snapshots = processSnapshots(measures);
    }
    return new Project(key.stringValue(), name.stringValue(), qualifier.stringValue(), snapshots);
  }

  private ArrayList<Snapshot> processSnapshots(JSONArray measures) {
    JSONValue jsonValue;
    JSONObject jsStock;
    JSONString date = null;
    JSONArray msr;
    ArrayList<Snapshot> snapshots = new ArrayList<Snapshot>();
    Map<String, Measure> projectMeasures = null;

    for (int i = 0; i < measures.size(); i++) {
      if ((jsStock = measures.get(i).isObject()) == null) continue;

      if ((jsonValue = jsStock.get("date")) != null) {
        date = jsonValue.isString();
      }


      if ((msr = jsStock.get("msr").isArray()) != null) {
        projectMeasures = processMeasures(msr);
      }

      snapshots.add(new Snapshot(date.stringValue(), projectMeasures));
    }

    return snapshots;
  }

  private Map<String, Measure> processMeasures(JSONArray msr) {
    JSONValue jsonValue;
    JSONObject jsMsr;
    JSONArray parameters;
    Map<String, Measure> projectMeasures = new HashMap<String, Measure>();
    for (int j = 0; j < msr.size(); j++) {
      if ((jsMsr = msr.get(j).isObject()) == null) continue;

      JSONString mkey, val;

      if ((jsonValue = jsMsr.get("metric")) == null) continue;
      if ((mkey = jsonValue.isString()) == null) continue;

      if ((jsonValue = jsMsr.get("value")) == null) continue;
      if ((val = jsonValue.isString()) == null) continue;

      Map<String, Parameter> measureParameters = null;
      if ((jsonValue = jsMsr.get("prm")) != null) {
        if ((parameters = jsonValue.isArray()) != null) {
          measureParameters = processMeasureParameters(parameters);
        }
      }

      projectMeasures.put(mkey.stringValue(), new Measure(mkey.stringValue(), val.stringValue(), measureParameters));
    }
    return projectMeasures;
  }

  private Map<String, Parameter> processMeasureParameters(JSONArray parameters) {
    JSONValue jsonValue;
    JSONObject jsParameters;

    Map<String, Parameter> measureParameters = new HashMap<String, Parameter>();
    for (int k = 0; k < parameters.size(); k++) {
      if ((jsParameters = parameters.get(k).isObject()) == null) continue;

      JSONString prmKey, prmVal;

      if ((jsonValue = jsParameters.get("key")) == null) continue;
      if ((prmKey = jsonValue.isString()) == null) continue;

      if ((jsonValue = jsParameters.get("val")) == null) continue;
      if ((prmVal = jsonValue.isString()) == null) continue;

      measureParameters.put(prmKey.stringValue(), new Parameter(prmKey.stringValue(), prmVal.stringValue(), null, null));
    }

    return measureParameters;
  }

}
