/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.testspage.client;

import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.http.client.URL;
import com.google.gwt.json.client.JSONArray;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import org.sonar.plugins.api.web.gwt.client.AbstractSourceViewer;
import org.sonar.plugins.api.web.gwt.client.JsonUtils;
import org.sonar.plugins.api.web.gwt.client.Utils;
import org.sonar.plugins.core.testspage.client.model.Measure;
import org.sonar.plugins.core.testspage.client.model.Project;
import org.sonar.plugins.core.testspage.client.widget.ClassListWidget;
import org.sonar.plugins.core.testspage.client.widget.CodeCoverageWidget;
import org.sonar.plugins.core.testspage.client.widget.ModuleListWidget;
import org.sonar.plugins.core.testspage.client.widget.PackageListWidget;
import org.sonar.plugins.core.testspage.client.widget.TestSuccessWidget;

import java.util.ArrayList;
import java.util.Map;

public class GwtTestsPage extends AbstractMeasuresPage {
  public static final String GWT_ID = "org.sonar.plugins.core.testspage.GwtTestsPage";
  private static final String METRICS_TO_RETRIEVE = "test_success_percentage,test_failures_count,test_errors_count,code_coverage,test_count,test_execution_time";

  private CodeCoverageWidget codeCoverageWidget = new CodeCoverageWidget();
  private TestSuccessWidget testSuccessWidget = new TestSuccessWidget();
  private ClassListWidget classList = new ClassListWidget();
  private PackageListWidget packageList = new PackageListWidget(classList);
  private ModuleListWidget moduleList = new ModuleListWidget(packageList);

  public void onModuleLoad() {
    //
    String urlModules = URL.encode(Utils.getServerUrl() + "/api/projects/" + getProjectKey() + "?includechildren=true&format=json&callback=");
    JsonUtils.requestJson(urlModules, new ModulesHandler());

    // Get data for blocks 'Code coverage' and 'Test success'
    String url = URL.encode(buildRestUrl(METRICS_TO_RETRIEVE)) + "&callback=";
    JsonUtils.requestJson(url, new BlocksHandler());

    //
    FlowPanel top = new FlowPanel();
    top.getElement().setId("block_modules");
    top.getElement().getStyle().setProperty("display", "none");
    top.add(codeCoverageWidget);
    top.add(testSuccessWidget);

    //
    HorizontalPanel bottom = new HorizontalPanel();
    bottom.setStyleName("gwt-breakdown");
    bottom.add(moduleList);
    bottom.add(packageList);
    bottom.add(classList);

    //
    FlowPanel main = new FlowPanel();
    main.add(top);
    main.add(bottom);

    HTML sourceViewer = new HTML("<div id='" + AbstractSourceViewer.HTML_ROOT_ID + "'></div>");
    main.add(sourceViewer);
    displayView(main);
  }

  private void updateBlock(ArrayList<Project> projects) {
    Map<String, Measure> measures = projects.get(0).getSnapshots().get(0).getMeasures();
    codeCoverageWidget.updateBlock(measures);
    testSuccessWidget.updateBlock(measures);
    DOM.getElementById("block_modules").getStyle().setProperty("display", "block");
  }

  private void updateModules(ArrayList<Project> projects) {
    //Map<String, Measure> measures = projects.get(0).getSnapshots().get(0).getMeasures();
    moduleList.updateBlock(projects);
  }

  public class ModulesHandler implements JsonUtils.JSONHandler {
    public void handleJSON(JavaScriptObject jso) {
      if (jso == null) {
        displayError("Couldn't retrieve JSON modules");
        return;
      }
      ArrayList<Project> projects = processResponse(new JSONArray(jso));
      updateModules(projects);
    }
  }

  public class BlocksHandler implements JsonUtils.JSONHandler {
    public void handleJSON(JavaScriptObject jso) {
      if (jso == null) {
        displayError("Couldn't retrieve JSON blocks");
        return;
      }
      ArrayList<Project> projects = processResponse(new JSONArray(jso));
      updateBlock(projects);
    }
  }

}
