/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.testspage.client.widget;

import com.google.gwt.i18n.client.NumberFormat;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.*;
import org.sonar.plugins.core.testspage.client.model.Measure;

import java.util.Map;

public class CodeCoverageWidget extends SonarWidget {

  private static final String BLOCK_ID = "code_coverage_";

  public CodeCoverageWidget() {
    // Block title
    Label title = new Label("Code coverage");
    title.setStyleName("h3 title");

    //
    Hyperlink percent_link = new Hyperlink("-", "");
    percent_link.getElement().setId(BLOCK_ID + "code_coverage");
    percent_link.setStyleName("big");
    Image percent_image = new Image(baseUrl + "/images/tendency/-2-black-small.png");
    FlowPanel lines = new FlowPanel();
    lines.setStyleName("line");
    lines.add(percent_link);
    lines.add(percent_image);

    //
    Hyperlink packages_link = new Hyperlink("-", "");
    packages_link.getElement().setId(BLOCK_ID + "test_count");
    HTML packages_label = new HTML("&nbsp;tests");
    Image tests_image = new Image(baseUrl + "/images/tendency/-2-black-small.png");
    FlowPanel packages = new FlowPanel();
    packages.setStyleName("line");
    packages.add(packages_link);
    packages.add(packages_label);

    // 'Execution time' line
    Hyperlink sec_link = new Hyperlink("-", "");
    sec_link.getElement().setId(BLOCK_ID + "test_execution_time");
    sec_link.getElement().setId(BLOCK_ID + "test_execution_time");
    Image sec_image = new Image(baseUrl + "/images/tendency/-2-black-small.png");
    FlowPanel sec = new FlowPanel();
    sec.setStyleName("line");
    sec.add(sec_link);
    sec.add(sec_image);

    // Build the block
    FlowPanel main = new FlowPanel();
    main.getElement().setId(BLOCK_ID);
    main.setStyleName("gwt-measures");
    main.add(title);
    main.add(lines);
    main.add(packages);
    main.add(sec);

    //
    initWidget(main);
  }

  public void updateBlock(Map<String, Measure> projectMeasures) {
    if (projectMeasures.containsKey("code_coverage")) {
      String val = projectMeasures.get("code_coverage").getVal();
//      NumberFormat nf = NumberFormat.getPercentFormat();
      double dVal = Double.parseDouble(val);
//      DOM.getElementById(BLOCK_ID + "code_coverage").getFirstChildElement().setInnerHTML(nf.format(dVal));
      DOM.getElementById(BLOCK_ID + "code_coverage").getFirstChildElement().setInnerHTML(NumberFormat.getFormat("#.#").format(dVal));
    }
    if (projectMeasures.containsKey("test_count")) {
      String val = projectMeasures.get("test_count").getVal();
      double dVal = Double.parseDouble(val);
      DOM.getElementById(BLOCK_ID + "test_count").getFirstChildElement().setInnerHTML(NumberFormat.getFormat("#").format(dVal));
    }
    if (projectMeasures.containsKey("test_execution_time")) {
      String val = projectMeasures.get("test_execution_time").getVal();
      double dVal = Double.parseDouble(val);
      DOM.getElementById(BLOCK_ID + "test_execution_time").getFirstChildElement().setInnerHTML(NumberFormat.getFormat("#").format(dVal));
    }
  }

}
