/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.testspage.client.widget;

import com.google.gwt.user.client.ui.*;
import com.google.gwt.user.client.DOM;
import org.sonar.plugins.core.testspage.client.model.Measure;

import java.util.Map;

public class ComplexityWidget extends SonarWidget {

  private static final String BLOCK_ID = "lines_of_code_";
  //ccn_class,ccn_function,ccn

  public ComplexityWidget() {
    // Block title
    Label title = new Label("Complexity");
    title.setStyleName("h3 title");

    // Methods
    Hyperlink lines_link = new Hyperlink("-", "");
    lines_link.getElement().setId(BLOCK_ID + "ccn_function");
    lines_link.setStyleName("big");
    HTML methods_label = new HTML("&nbsp;/method");
    FlowPanel methods = new FlowPanel();
    methods.setStyleName("line");
    methods.add(lines_link);
    methods.add(methods_label);

    // Classes
    Hyperlink classes_link = new Hyperlink("-", "");
    classes_link.getElement().setId(BLOCK_ID + "ccn_class");
    HTML classes_label = new HTML("&nbsp;/class");
    FlowPanel classes = new FlowPanel();
    classes.setStyleName("line");
    classes.add(classes_link);
    classes.add(classes_label);

    // Decision points
    Hyperlink decision_link = new Hyperlink("-", "");
    decision_link.getElement().setId(BLOCK_ID + "ccn");
    Image decision_image = new Image(baseUrl + "/images/tendency/2-black-small.png");
    HTML decision_label = new HTML("&nbsp;decision points");
    FlowPanel decision = new FlowPanel();
    decision.setStyleName("line");
    decision.add(decision_link);
    decision.add(decision_image);
    decision.add(decision_label);

    // Build the data block
    FlowPanel dataPanel = new FlowPanel();
    dataPanel.setStyleName("gwt-measures");
    dataPanel.add(title);
    dataPanel.add(methods);
    dataPanel.add(classes);
    dataPanel.add(decision);

    // Build the graph block
    Image complexity_chart_image = new Image(baseUrl + "/charts/complexity_distribution_chart?id=1");
    complexity_chart_image.setStyleName("gwt-complexity-chart");

    // Build the entire block
    HorizontalPanel main = new HorizontalPanel();
    main.add(dataPanel);
    main.add(complexity_chart_image);

    //
    initWidget(main);
  }

  public void updateBlock(Map<String, Measure> projectMeasures) {
    if (projectMeasures.containsKey("ccn")) {
      String val = projectMeasures.get("ccn").getVal();
      DOM.getElementById(BLOCK_ID + "ccn").getFirstChildElement().setInnerHTML(val);
    }
    if (projectMeasures.containsKey("ccn_function")) {
      String val = projectMeasures.get("ccn_function").getVal();
      DOM.getElementById(BLOCK_ID + "ccn_function").getFirstChildElement().setInnerHTML(val);
    }
    if (projectMeasures.containsKey("ccn_class")) {
      String val = projectMeasures.get("ccn_class").getVal();
      DOM.getElementById(BLOCK_ID + "ccn_class").getFirstChildElement().setInnerHTML(val);
    }
  }

}
