/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.testspage.client.widget;

import com.google.gwt.user.client.ui.*;
import com.google.gwt.user.client.DOM;
import org.sonar.plugins.core.testspage.client.model.Measure;

import java.util.Map;

public class LinesOfCodeWidget extends SonarWidget {

  private static final String JSON_URL = "ncss,packages_count,classes_count,functions_count,duplicated_lines_ratio";
  private static final String BLOCK_ID = "lines_of_code_";

  public LinesOfCodeWidget() {
    // Block title
    Label title = new Label("Lines of code");
    title.setStyleName("h3 title");

    // Ncss
    Hyperlink lines_link = new Hyperlink("-", "");
    lines_link.getElement().setId(BLOCK_ID + "ncss");
    lines_link.setStyleName("big");
    Image lines_image = new Image("http://localhost:9000/dev/images/tendency/2-black.png");
    FlowPanel lines = new FlowPanel();
    lines.setStyleName("line");
    lines.add(lines_link);
    lines.add(lines_image);

    // Packages
    HTML packages_label = new HTML("&nbsp;packages");
    Hyperlink packages_link = new Hyperlink("-", "");
    packages_link.getElement().setId(BLOCK_ID + "packages_count");
    FlowPanel packages = new FlowPanel();
    packages.setStyleName("line");
    packages.add(packages_link);
    packages.add(packages_label);

    // Classes
    Hyperlink classes_link = new Hyperlink("-", "");
    classes_link.getElement().setId(BLOCK_ID + "classes_count");
    HTML classes_label = new HTML("&nbsp;classes");
    FlowPanel classes = new FlowPanel();
    classes.setStyleName("line");
    classes.add(classes_link);
    classes.add(classes_label);

    // Methods
    Hyperlink methods_link = new Hyperlink("-", "");
    methods_link.getElement().setId(BLOCK_ID + "functions_count");
    HTML methods_label = new HTML("&nbsp;methods");
    FlowPanel methods = new FlowPanel();
    methods.setStyleName("line");
    methods.add(methods_link);
    methods.add(methods_label);

    // Duplicated lines
    Hyperlink dup_link = new Hyperlink("-", "");
    dup_link.getElement().setId(BLOCK_ID + "duplicated_lines_ratio");
    HTML dup_label = new HTML("&nbsp;duplicated lines");
    FlowPanel dup = new FlowPanel();
    dup.setStyleName("line");
    dup.add(dup_link);
    dup.add(dup_label);

    // Build the block
    FlowPanel main = new FlowPanel();
    main.setStyleName("gwt-measures");
    main.add(title);
    main.add(lines);
    main.add(packages);
    main.add(classes);
    main.add(methods);
    main.add(dup);

    //
    initWidget(main);
  }

  public void updateBlock(Map<String, Measure> projectMeasures) {
    if (projectMeasures.containsKey("ncss")) {
      String val = projectMeasures.get("ncss").getVal();
      DOM.getElementById(BLOCK_ID + "ncss").getFirstChildElement().setInnerHTML(val);
    }
    if (projectMeasures.containsKey("packages_count")) {
      String val = projectMeasures.get("packages_count").getVal();
      DOM.getElementById(BLOCK_ID + "packages_count").getFirstChildElement().setInnerHTML(val);
    }
    if (projectMeasures.containsKey("classes_count")) {
      String val = projectMeasures.get("classes_count").getVal();
      DOM.getElementById(BLOCK_ID + "classes_count").getFirstChildElement().setInnerHTML(val);
    }
    if (projectMeasures.containsKey("functions_count")) {
      String val = projectMeasures.get("functions_count").getVal();
      DOM.getElementById(BLOCK_ID + "functions_count").getFirstChildElement().setInnerHTML(val);
    }
    if (projectMeasures.containsKey("duplicated_lines_ratio")) {
      String val = projectMeasures.get("duplicated_lines_ratio").getVal();
      DOM.getElementById(BLOCK_ID + "duplicated_lines_ratio").getFirstChildElement().setInnerHTML(val);
    }
  }

}
