/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.testspage.client.widget;

import com.google.gwt.user.client.ui.*;
import org.sonar.plugins.core.testspage.client.model.Project;

import java.util.ArrayList;

public class ModuleListWidget extends SonarWidget implements ClickListener {

  private FlexTable tableModules = new FlexTable();
  private int selectedRow = -1;
  private PackageListWidget packageList;

  public ModuleListWidget(PackageListWidget packageList) {
    this.packageList = packageList;
    init();
  }

  public ModuleListWidget() {
    init();
  }

  private void init() {
    // 
    Label label = new Label("Modules");
    label.setStyleName("gwt-breakdown-title");

    // Setup the table.
    tableModules.setStyleName("items");
    //tableModules.addTableListener(this);

    ScrollPanel scrollPanel = new ScrollPanel(tableModules);
    scrollPanel.setStyleName("scrollable");

    //
    FlowPanel main = new FlowPanel();
    main.add(label);
    main.add(scrollPanel);

    initWidget(main);
  }

  public void onCellClicked(SourcesTableEvents sender, int row, int cell) {
    System.out.println("ModuleListWidget onCellClicked");
    selectRow(row);
    packageList.update(row + 1);
  }

  public void onClick(Widget widget) {
    System.out.println("ModuleListWidget onClick : " + widget);
    int nb = Integer.parseInt(widget.getElement().getId().replace("sid_", ""));
    selectRow(nb);
    packageList.update(nb + 1);
  }

  private void selectRow(int row) {
    styleRow(selectedRow, false);
    styleRow(row, true);
    selectedRow = row;
  }

  private void styleRow(int row, boolean selected) {
    if (row != -1) {
      if (selected) {
        tableModules.getRowFormatter().addStyleName(row, "selected");
      } else {
        tableModules.getRowFormatter().removeStyleName(row, "selected");
      }
    }
  }

  public void updateBlock(ArrayList<Project> projects) {
    tableModules.clear();

    int i = 0;
    for (Project project : projects) {
      // Image zoom
      Image image = new Image(baseUrl + "/images/zoom.png");
      tableModules.setWidget(i, 0, image);
      tableModules.getCellFormatter().setWordWrap(i, 0, false);
      tableModules.getCellFormatter().setStyleName(i, 0, "zoom");

      // Module name
      Hyperlink link = new Hyperlink(project.getName(), "");
      link.getElement().setId("sid_" + i);
      link.addClickListener(this);
      tableModules.setWidget(i, 1, link);
      tableModules.getCellFormatter().setWordWrap(i, 1, false);

      // Number of lines
      String metricValue = "";
      if (project.getSnapshots() != null && project.getSnapshots().size() > 0 && project.getSnapshots().get(0).getMeasures() != null && project.getSnapshots().get(0).getMeasures().size() > 0) {
        metricValue = project.getSnapshots().get(0).getMeasures().get(0).getVal();
      }
      tableModules.setText(i, 2, metricValue);
      tableModules.getCellFormatter().setWordWrap(i, 2, false);
      tableModules.getCellFormatter().setStyleName(i, 2, "right");

      // Row style
      tableModules.getRowFormatter().setStyleName(i, i % 2 == 0 ? "even" : "odd");
      tableModules.getRowFormatter().addStyleName(i, "hoverable");

      if (i == 0) {
        tableModules.getRowFormatter().addStyleName(i, "selected");
        selectedRow = 0;
      }

      i++;
    }
  }

}
