/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.testspage.client.widget;

import com.google.gwt.user.client.ui.*;

public class PackageListWidget extends SonarWidget implements TableListener, ClickListener {

  private FlexTable tablePackages = new FlexTable();
  private int selectedRow = -1;
  private ClassListWidget classList;

  public PackageListWidget() {
    init();
  }

  public PackageListWidget(ClassListWidget classList) {
    this.classList = classList;
    init();
  }

  private void init() {
    //
    Label label = new Label("Packages");
    label.setStyleName("gwt-breakdown-title");

    // Setup the table.
    tablePackages.setStyleName("items");
    tablePackages.addTableListener(this);
    ScrollPanel scrollPanel = new ScrollPanel(tablePackages);
    scrollPanel.setStyleName("scrollable");

    // Build the block
    FlowPanel main = new FlowPanel();
    main.setStyleName("gwt-separator");
    main.add(label);
    main.add(scrollPanel);

    initTable(1);

    initWidget(main);
  }

  private void initTable(int coef) {
    // 
    for (int i = 0; i < 100; ++i) {
      // Image zoom
      Image image = new Image(baseUrl + "/images/zoom.png");
      tablePackages.setWidget(i, 0, image);
      tablePackages.getCellFormatter().setWordWrap(i, 0, false);
      tablePackages.getCellFormatter().setStyleName(i, 0, "zoom");

      // Package name
      tablePackages.setText(i, 1, "Package " + (i * coef));
      tablePackages.getCellFormatter().setWordWrap(i, 1, false);

      // 
      tablePackages.setText(i, 2, "12'258");
      tablePackages.getCellFormatter().setWordWrap(i, 2, false);
      tablePackages.getCellFormatter().setStyleName(i, 2, "right");

      //
      tablePackages.getRowFormatter().setStyleName(i, i % 2 == 0 ? "even" : "odd");
      tablePackages.getRowFormatter().addStyleName(i, "hoverable");

      if (i == 0) {
        tablePackages.getRowFormatter().addStyleName(i, "selected");
        selectedRow = 0;
      }
    }
  }

  public void update(int coef) {
    System.out.println("PackageListWidget update");
    classList.hide();
    initTable(coef);
  }

  public void onCellClicked(SourcesTableEvents sender, int row, int cell) {
    System.out.println("PackageListWidget onCellClicked");
    selectRow(row);
    classList.update(row + 1);
  }

  public void onClick(Widget widget) {
    //To change body of implemented methods use File | Settings | File Templates.
  }

  private void selectRow(int row) {
    styleRow(selectedRow, false);
    styleRow(row, true);
    selectedRow = row;
  }

  private void styleRow(int row, boolean selected) {
    if (row != -1) {
      if (selected) {
        tablePackages.getRowFormatter().addStyleName(row, "selected");
      } else {
        tablePackages.getRowFormatter().removeStyleName(row, "selected");
      }
    }
  }

}