/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.unittests;

import ch.hortis.sonar.model.Metric;
import org.sonar.commons.measures.Measure;
import org.sonar.plugins.api.Language;
import org.sonar.plugins.api.jobs.AbstractJob;
import org.sonar.plugins.api.jobs.Resource;
import org.sonar.plugins.api.metrics.CoreMetrics;

import java.util.Arrays;
import java.util.List;

public class ProjectCodeCoverageJob extends AbstractJob {

  @Override
  public List<Metric> generatesMetrics() {
    return Arrays.asList(CoreMetrics.CODE_COVERAGE);
  }

  protected Language shouldExecuteOnLanguage() {
    return null;
  }

  public boolean shouldExecuteOnResource(Resource resource) {
    return resource.isProject();
  }

  public void execute(Resource resource) {
    if (resource.getMeasure(CoreMetrics.CODE_COVERAGE) == null) {
      Double sumLines = 0.0;
      Double sumCcWeighted = 0.0;
      for (Resource child : resource.getChildren()) {
        Measure nbLines = child.getMeasure(CoreMetrics.NLOC);
        Measure cc = child.getMeasure(CoreMetrics.CODE_COVERAGE);
        if (nbLines != null && cc != null) {
          sumLines += nbLines.getValue();
          sumCcWeighted += cc.getValue() * nbLines.getValue();
        }
      }
      if (sumLines > 0.0 && sumCcWeighted >= 0.0) {
        resource.addMeasure(CoreMetrics.CODE_COVERAGE, sumCcWeighted / sumLines);
      }
    }
  }
}
