/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.unittests;

import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.service.MeasureKey;
import org.sonar.commons.measures.Measure;
import org.sonar.plugins.api.jobs.AbstractJob;
import org.sonar.plugins.api.jobs.Resource;
import org.sonar.plugins.api.metrics.CoreMetrics;
import org.sonar.plugins.api.Language;

import java.util.Arrays;
import java.util.List;

public class UnitTestsConsolidationJob extends AbstractJob {

  @Override
  public List<Metric> generatesMetrics() {
    return Arrays.asList(CoreMetrics.TESTS_TIME, CoreMetrics.TESTS_COUNT, CoreMetrics.TESTS_ERRORS, CoreMetrics.TESTS_FAILURES);
  }

  public boolean shouldExecuteOnProject(Resource project) {
    return true;
  }

  protected Language shouldExecuteOnLanguage() {
    return null; // all languages
  }

  public boolean shouldExecuteOnResource(Resource node) {
    return node.isJavaTest() || !node.isClass();
  }

  public void execute(Resource resource) {
    sumChildren(resource, CoreMetrics.TESTS_TIME);
    Double tests = sumChildren(resource, CoreMetrics.TESTS_COUNT);
    Double errors = sumChildren(resource, CoreMetrics.TESTS_ERRORS);
    Double failures = sumChildren(resource, CoreMetrics.TESTS_FAILURES);

    if (isPositive(tests, true) && isPositive(errors, false) && isPositive(failures, false)) {
      Double errorsAndFailuresRatio = (errors + failures) * 100.0 / tests;
      resource.addMeasure(CoreMetrics.TESTS_SUCCESS_PERCENTAGE, 100.0 - errorsAndFailuresRatio);
    }
  }

  private boolean isPositive(Double d, boolean strict) {
	if (d!=null) {
		return (strict ? d>0.0 : d>=0.0);
	}
    return false;
  }

  private Double sumChildren(Resource resource, Metric metric) {
    List<Measure> childrenMeasures = resource.getChildrenMeasures(new MeasureKey(metric));
    if (childrenMeasures != null && childrenMeasures.size() > 0) {
      Double sum = 0.0;
      boolean hasChildrenMeasures = false;
      for (Measure measure : childrenMeasures) {
        if (measure.getValue() != null) {
          sum += measure.getValue();
          hasChildrenMeasures = true;
        }
      }
      if (hasChildrenMeasures) {
        resource.addMeasure(metric, sum);
        return sum;
      }
    }
    return null;
  }
}
