/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.violationssourceviewer.client;

import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.http.client.URL;
import com.google.gwt.json.client.JSONArray;
import com.google.gwt.json.client.JSONNumber;
import com.google.gwt.json.client.JSONObject;
import com.google.gwt.json.client.JSONString;
import org.sonar.plugins.api.web.gwt.client.AbstractSourceViewer;
import org.sonar.plugins.api.web.gwt.client.JsonUtils;
import org.sonar.plugins.api.web.gwt.client.LineDecorator;
import org.sonar.plugins.api.web.gwt.client.Utils;
import org.sonar.plugins.core.violationssourceviewer.client.model.Violation;
import org.sonar.plugins.core.violationssourceviewer.client.model.ViolationsByLine;
import org.sonar.plugins.core.violationssourceviewer.client.widget.ViolationsWidget;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class GwtViolationsSourceViewer extends AbstractSourceViewer {
  public static final String GWT_ID ="org.sonar.plugins.core.violationssourceviewer.GwtViolationsSourceViewer";

  protected String getGwtId() {
    return GWT_ID;
  }

  protected boolean showValueColumn() {
    return true;
  }

  protected void requestLineDecorators(String sourceKey) {
    String url = URL.encode(Utils.getServerUrl() + "/api/projects/" + sourceKey + "/violations?format=json&callback=");
    JsonUtils.requestJson(url, new LineDecoratorsHandler());
  }

  public class LineDecoratorsHandler implements JsonUtils.JSONHandler {

    public void handleJSON(JavaScriptObject obj) {
      if (obj == null) {
        return;
      }
      ViolationsByLine violationsByLine = new ViolationsByLine();
      JSONArray jsonArray = new JSONArray(obj);
      for (int i = 0; i < jsonArray.size(); i++) {
        JSONObject jsViolation = jsonArray.get(i).isObject();
        if (jsViolation == null) continue;
        JSONString rule = jsViolation.get("rule").isString();
        JSONString message = jsViolation.get("message").isString();
        JSONNumber line = jsViolation.get("line").isNumber();
        JSONString level = jsViolation.get("level").isString();
        Integer intLine = Integer.parseInt(line.toString());
        Violation violation = new Violation(message.stringValue(), rule.stringValue(), null, level.stringValue());
        violationsByLine.add(intLine, violation);
      }

      Map<Integer, LineDecorator> result = new HashMap<Integer, LineDecorator>();
      for (Integer line : violationsByLine.getLines()) {
        List<Violation> violations = violationsByLine.getViolations(line);
        String value = String.valueOf(violations.size());
        result.put(line, new LineDecorator(null, value, null, new ViolationsWidget(violations)));
      }
      setDecorators(result);
      setDecoratorsRetrieved(true);
      refreshLines();
    }
  }

  protected void exportJavascript() {
    exportNativeJavascript(this);
  }

  public static native void exportNativeJavascript(GwtViolationsSourceViewer obj) /*-{
    $wnd.load_org_sonar_plugins_core_violationssourceviewer_GwtViolationsSourceViewer = function() {
      obj.@org.sonar.plugins.core.violationssourceviewer.client.GwtViolationsSourceViewer::loadSource()();
    }
    }-*/;
}